package cn.iocoder.yudao.module.system.controller.admin.information.vo;

import com.alibaba.excel.annotation.ExcelProperty;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.*;

import java.math.BigDecimal;
import java.util.*;
import javax.validation.constraints.*;
import org.springframework.format.annotation.DateTimeFormat;
import java.time.LocalDateTime;

@Schema(description = "管理后台 - 设备信息新增/修改 Request VO")
@Data
public class DeviceInformationSaveReqVO {

    @Schema(description = "主键ID", requiredMode = Schema.RequiredMode.REQUIRED, example = "14335")
    private Long id;

    @Schema(description = "地图id", example = "6609")
    private Long positionMapId;

    @Schema(description = "地图子表节点id", example = "23591")
    private Long positionMapItemId;

    @Schema(description = "库位坐标x轴")
    private String locationX;

    @Schema(description = "库位坐标y轴")
    private String locationY;

    @Schema(description = "宽度")
    private BigDecimal locationWide;

    @Schema(description = "深度")
    private BigDecimal locationDeep;

    //1:充电桩,2:输送线,3:码垛机,4:自动门,5:提升机,6:信号灯,7:按钮盒,8:拆垛机
    @Schema(description = "设备类型 字典device_type", requiredMode = Schema.RequiredMode.REQUIRED, example = "2")
    @NotNull(message = "设备类型不能为空")
    private Integer deviceType;

    @Schema(description = "设备编号")
    @Size(min = 0, max = 50 ,message = "设备编号长度超过限制")
    private String deviceNo;

    @Schema(description = "设备位置")
    private String deviceLocation;

    @Schema(description = "mac地址")
    @Size(min = 0, max = 50 ,message = "Mac地址长度超过限制")
    private String macAddress;

    @Schema(description = "设备在地图上图标")
    private String mapImageUrl;

    @Schema(description = "上传图片附件", requiredMode = Schema.RequiredMode.REQUIRED, example = "https://www.iocoder.cn")
    private String url;

    @Schema(description = "图片设置（1：默认图片、2：上传图片、 3：不显示图片）")
    private Integer pictureConfig;

    @Schema(description = "设备状态（1：在线、2：离线、 3：异常）", example = "1")
    private Integer deviceStatus;

    @Schema(description = "设备启用禁用（0：禁用、1：启用）")
    private Integer deviceEnable;

    @Schema(description = "设备协议")
    private String deviceAgreement;

    @Schema(description = "设备最后通讯时间")
    private LocalDateTime deviceLastTime;

    @Schema(description = "设备使用状态（0:空闲、1:使用中）")
    private Integer deviceUseStatus;

    @Schema(description = "设备专有属性（1：自动充电类型充电桩、2：手动充电类型充电桩）")
    private Integer deviceAttribute;

    @Schema(description = "设备IP")
    @Size(min = 0, max = 20 ,message = "设备IP长度超过限制")
    private String deviceIp;

    @Schema(description = "端口")
    @Size(min = 0, max = 10 ,message = "端口长度超过限制")
    private String devicePort;

    @Schema(description = "最后使用者")
    private String lastUser;

}
