package cn.iocoder.yudao.module.system.service.robot;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.module.system.controller.admin.robot.detail.RobotTaskDetailLogResoVO;
import cn.iocoder.yudao.module.system.controller.admin.robot.detail.RobotTaskDetailLogVO;
import cn.iocoder.yudao.module.system.controller.admin.robot.vo.RobotTaskDetailAddVO;
import cn.iocoder.yudao.module.system.controller.admin.robot.vo.RobotTaskPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.robot.vo.RobotTaskRespVO;
import cn.iocoder.yudao.module.system.controller.admin.robot.vo.RobotTaskSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotTaskDO;
import com.baomidou.mybatisplus.extension.service.IService;

/**
 * 机器人任务主表 Service 接口
 *
 * @author 陈宾顺
 */
public interface RobotTaskService extends IService<RobotTaskDO> {

    /**
     * 创建机器人任务主表
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createTask(@Valid RobotTaskSaveReqVO createReqVO) throws InterruptedException;

    /**
     * 更新机器人任务主表
     *
     * @param updateReqVO 更新信息
     */
    void updateTask(@Valid RobotTaskSaveReqVO updateReqVO);

    /**
     * 删除机器人任务主表
     *
     * @param id 编号
     */
    void deleteTask(Long id);

    /**
     * 获得机器人任务主表
     *
     * @param id 编号
     * @return 机器人任务主表
     */
    RobotTaskDO getTask(Long id);

    /**
     * 获得机器人任务主表分页
     *
     * @param pageReqVO 分页查询
     * @return 机器人任务主表分页
     */
    PageResult<RobotTaskRespVO> getTaskPage(RobotTaskPageReqVO pageReqVO);

    /**
     * 获取任务号
     *
     * @return
     */
    String getTaskNo();

    /**
     * PP分配的任务
     * @param message
     */
//    void ppDistributionTask(String message);

    /**
     * 任务日志分页
     *
     * @param pageReqVO
     * @return
     */
    PageResult<RobotTaskDetailLogResoVO> logPage(RobotTaskDetailLogVO pageReqVO);

    /**
     * 添加循环任务点位
     *
     * @param id
     * @param taskDetailList
     */
    void addCycle(Long id, List<RobotTaskDetailAddVO> taskDetailList);

    void closeTask(@Valid RobotTaskSaveReqVO updateReqVO);

    /**
     * PP分配任务
     *
     * @param message
     */
    void assignTasks(String message);

    /**
     * 发送关闭任务
     *
     * @param id
     * @param mac
     */
    void closeTaskDetail(String id, String mac, String orderType);

    /**
     * 校验是否存在未完成的任务
     */
    void checkHaveDoingTask();

    /**
     * 设置车辆充电完成
     * @param robotNo
     */
    void chargeDone(String robotNo);
}
